@@ -5,7 +5,7 @@
 #' @param ... other arguments passed on to methods
 #' @keywords internal
 #' @export
-#' @seealso \code{\link[plyr]{round_any}}
+#' @seealso [plyr::round_any()]
 fullseq <- function(range, size, ...) UseMethod("fullseq")
 
 #' @export
@@ -33,5 +33,13 @@ fullseq.Date <- function(range, size, ...) {
 }
 #' @export
 fullseq.POSIXt <- function(range, size, ...) {
-  seq(floor_time(range[1], size), ceiling_time(range[2], size), by = size)
+
+  # for subsecond interval support
+  # seq() does not support partial secs in character strings
+  parsed <- parse_unit_spec(size)
+  if (parsed$unit == "sec") {
+    seq(floor_time(range[1], size), ceiling_time(range[2], size), by = parsed$mult)
+  } else {
+    seq(floor_time(range[1], size), ceiling_time(range[2], size), by = size)
+  }
 }
